@@ -48,13 +48,24 @@ test_that("common_samples succeeds on matrices with no or partial dimension name
                     default_names    = TRUE))
 })
 
-test_that("common_samples fails on matrices with dimension names but no common samples",  {
-  expect_error(
+# test_that("common_samples fails on matrices with dimension names but no common samples",  {
+#   expect_error(
+#     suppressWarnings(
+#       common_samples(
+#         `rownames<-`(counts, paste0("Sample_", 1:49)),
+#         covariates))#,
+#     #"Conflicting samples names in count matrix and covariates data frames"
+#   )
+# })
+
+cli::test_that_cli("common_samples fails on matrices with dimension names but conflicting names",  {
+  testthat::local_edition(3)
+  expect_snapshot(
+    error = TRUE,
     suppressWarnings(
       common_samples(
         `rownames<-`(counts, paste0("Sample_", 1:49)),
-        covariates)),
-    "Conflicting samples names in count matrix and covariates data frames"
+        covariates))
   )
 })
 
@@ -65,9 +76,10 @@ test_that("common_samples succeeds on matrices with dimension names and differen
                     default_names    = FALSE))
 })
 
-test_that("common_samples find biggest subset of common samples and produces message.",  {
-  expect_message(result <- common_samples(counts, covariates[1:35, ]),
-                 "14 samples were dropped from the abundance matrix for lack of associated covariates.")
+cli::test_that_cli("common_samples find biggest subset of common samples and produces message.",  {
+  testthat::local_edition(3)
+  expect_snapshot(result <- common_samples(counts, covariates[1:35, ]))
+  expect_warning(result <- common_samples(counts, covariates[1:35, ]))
   expect_length(result$common_samples, 35)
   expect_equal(result$common_samples,
                as.character(1:35))
@@ -156,14 +168,15 @@ test_that("compute_offset provides correct answers for identical samples", {
   expect_null(compute_offset(counts, "none"))
 })
 
-test_that("compute_offset fails with an informative error when given a data.frame", {
+cli::test_that_cli("compute_offset fails with an informative error when given a data.frame", {
   sizes <- rep(1, 5)
   counts <- sizes %o% 1:10
-
-  expect_error(compute_offset(counts, data.frame(counts)),
-               regexp = "You supplied a data.frame to compute_offset(). Did you mean to supply a numeric matrix?
-  Try converting your data.frame to a matrix with as.matrix().",
-               fixed = TRUE)
+  testthat::local_edition(3)
+  testthat::expect_snapshot(error = TRUE,
+                            compute_offset(counts, data.frame(counts)))
+#               regexp = "You supplied a data.frame to compute_offset(). Did you mean to supply a numeric matrix?
+#  Try converting your data.frame to a matrix with as.matrix().",
+#                fixed = TRUE)
 
 })
 
@@ -369,21 +382,22 @@ test_that("species_variance works as intented", {
 
 test_that("prepare_data drops samples with no positive counts", {
   counts[1:2, ] <- 0
-  expect_warning(result <- prepare_data(counts, covariates, offset = "none"),
-                 "Sample(s) 1 and 2 dropped for lack of positive counts.",
-                 fixed = TRUE
+  expect_warning(result <- prepare_data(counts, covariates, offset = "none")#,
+             #    "Sample(s) 1 and 2 dropped for lack of positive counts.",
+            #     fixed = TRUE
                  )
   expect_equal(dim(result), c(nrow(counts)-2, ncol(covariates)+1))
   expect_equal(dim(result$Abundance), c(nrow(counts)-2, ncol(counts)))
 })
 
-test_that("prepare_data replace NA with 0s", {
-  counts[1:2, 1] <- NA
-  expect_warning(result <- prepare_data(counts, covariates, offset = "none"),
-                 "NA values in count table replaced with 0.")
-  expect_equal(dim(result), c(nrow(counts), ncol(covariates)+1))
-  expect_equal(dim(result$Abundance), c(nrow(counts), ncol(counts)))
-})
+ test_that("prepare_data replace NA with 0s", {
+   counts[1:2, 1] <- NA
+   expect_warning(result <- prepare_data(counts, covariates, offset = "none"))
+   #,
+#                  "NA values in count table replaced with 0.")
+   expect_equal(dim(result), c(nrow(counts), ncol(covariates)+1))
+   expect_equal(dim(result$Abundance), c(nrow(counts), ncol(counts)))
+ })
 
 result <- data.frame(
   Abundance = NA,
